@@ -20,7 +20,8 @@
     lidR.interactive = TRUE,
     lidR.debug = FALSE,
     lidR.buildVRT = TRUE,
-    lidR.check.nested.parallelism = TRUE)
+    lidR.check.nested.parallelism = TRUE,
+    lidR.optimize.voxel.rendering = TRUE)
 
   max <- R_omp_get_max_threads()
   if (max > 0)
