@@ -18,3 +18,12 @@ test_that("functions throw error were needed", {
   expect_error(decimate_points(ctg, homogenize(5)), "output file")
 })
 
+test_that("User cannot use one of readXXXLAS in catalog_apply", {
+  myfun <- function(cluster, ...) {
+    las <- readTLSLAS(cluster)
+    if (is.empty(las)) return(NULL)
+    return(0)
+  }
+
+  expect_error(catalog_apply(megaplot_ctg, myfun), "Use readLAS")
+})
